/* DEFS.H */

/* Copyright 1992 Digital Dynamics, All rights reserved. */

/************************************************************/
/* Constants, macros, and typedefs for use ONLY with SBBS	*/
/************************************************************/

#ifndef _DEFS_H
#define _DEFS_H

#include <time.h>

/*************/
/* Constants */
/*************/

#define DEBUG		0		/* Set to zero to stop DEBUG LOG writes 	*/
#define INHOUSE 	1		/* Set to non-zero for In-house devlopement */

#define VERSION 	"1c"    /* Version, Major number, Minor letter */
#define REVISION	2		/* Revision number */
#define BETA		1		/* Beta release? */

/*********************/
/* OS specific stuff */
/*********************/

#if __OS2__

/* key words */

#define far
#define huge

/* my functions */

#define mswait(x) useint(x)
#define ivhctl(x) useint(x)
#define rioini(x,y) useints(x,y)
#define incom() retint()
#define outcom(x) useint(x)
#define dtr(x) useint(x)
#define setbaud(x) useint(x)
#define rioctl(x) useint(x)
#define lclwx() wherex()
#define lclwy() wherey()
#define lclxy(x,y) gotoxy(x,y)
#define lputc(x) putch(x)
#define lputs(x) cputs(x)
#define lclini(x) useint(x)
#define lclaes() retint()
#define lread(f,b,l) read(f,b,l)
#define lwrite(f,b,l) write(f,b,l)

/* library functions */

#define setcbrk(x) useint(x)
#define sound(x) useint(x)
#define delay(x) useint(x)
#define nosound()

#endif

/************/
/* Maximums */
/************/

#define MAX_NODES		250
#define MAX_USERS	   9000 /* Maximum number of users on system		*/
#define MAX_GRPS		500 /* Maximum total number of message groups	*/
#define MAX_SUBS	  20000 /* Maximum total number of message subs 	*/
#define MAX_LIBS		500 /* Maximum total number of file libraries	*/
#define MAX_DIRS	  20000 /* Maximum total number of file directories */
#define MAX_POSTS	   2000 /* Maximum number of posts per sub			*/
#define MAX_FILES	   2000 /* Maximum number of files per dir			*/
#define MAX_MAILW	   2000 /* Maximum number of emails to scan 		*/
#define MAX_MAILS	   2000 /* Maximum number of email sent to scan 	*/
#define MAX_SYSMAIL    5000 /* Maximum number of total emails in system */
#define MAX_TXTSECS 	500 /* Maximum number of text file sections 	*/
#define MAX_TXTFILES	500 /* Maximum number of text files per section */
#define MAX_BATUP		500 /* Maximum number of batch uploads			*/
#define MAX_BATDN		500 /* Maximum number of batch dnloads			*/
#define MAX_XTRNS		500 /* Maximum number of external programs		*/
#define MAX_RESULTS 	500 /* Maximum number of modem result codes 	*/
#define MAX_LINES		500 /* Maximum number of lines in a message 	*/
#define MAX_PROTS		250 /* Maximum number of transfer protocols 	*/
#define MAX_FEXTRS		500 /* Maximum number of extractable file types */
#define MAX_FVIEWS		500 /* Maximum number of viewable file types	*/
#define MAX_FTESTS		500 /* Maximum number of testable file types	*/
#define MAX_USERXFER	500 /* Maximum number of dest. users of usrxfer */
#define MAX_XEDITS		500 /* Maximum number of external editors		*/
#define MAX_ALTPATHS	250 /* Maximum number of alternate file paths	*/
#define MAX_QHUBS		500 /* Maximum number of QWK Network hubs		*/
#define MAX_QSUBS		500 /* Maximum number of Sub-boards per hub 	*/
#define MAX_ALTCMDS 	100 /* Maximum number of Alternate command sets */
#define MAX_PHUBS		500 /* Maximum number of PostLink/PCRelay hubs	*/
#define MAX_CHANS		500 /* Maximum number of MultiChat channels 	*/
#define MAX_PAGES		100 /* Maximum number of External chat pagers	*/
#define MAX_CHATACTS	500 /* Maximum number of Chat action commands	*/


#ifdef SCFG
#define LEN_DIR 256
#else
#define LEN_DIR 42
#endif

							/* Lengths of various strings				*/
#define LEN_GSNAME	15		/* Group/Lib short name						*/
#define LEN_GLNAME	40		/* Group/Lib long name						*/
#define LEN_SSNAME	25		/* Sub/Dir short name						*/
#define LEN_SLNAME	40		/* Sub/Dir long name						*/

								/* User Questions						*/
#define UQ_ALIASES	(1L<<0) 	/* Ask for alias						*/
#define UQ_LOCATION (1L<<1) 	/* Ask for location 					*/
#define UQ_ADDRESS	(1L<<2) 	/* Ask for address						*/
#define UQ_PHONE	(1L<<3) 	/* Ask for phone number 				*/
#define UQ_HANDLE	(1L<<4) 	/* Ask for chat handle					*/
#define UQ_DUPHAND	(1L<<5) 	/* Search for duplicate handles 		*/
#define UQ_SEX		(1L<<6) 	/* Ask for sex :)						*/
#define UQ_BIRTH	(1L<<7) 	/* Ask for birth date					*/
#define UQ_COMP 	(1L<<8) 	/* Ask for computer type				*/
#define UQ_MC_COMP	(1L<<9) 	/* Multiple choice computer type		*/
#define UQ_REALNAME (1L<<10)	/* Ask for real name					*/
#define UQ_DUPREAL	(1L<<11)	/* Search for duplicate real names		*/
#define UQ_COMPANY	(1L<<12)	/* Ask for company name 				*/
#define UQ_NOEXASC	(1L<<13)	/* Don't allow ex-ASCII in user text    */


								/* Different bits in sys_misc				*/
#define SM_CLOSED	(1L<<0) 	/* System is clsoed to New Users			*/
#define SM_SYSSTAT	(1L<<1) 	/* Sysops activity included in statistics	*/
#define SM_NOBEEP	(1L<<2) 	/* No beep sound locally					*/
#define SM_PWEDIT	(1L<<3) 	/* Allow users to change their passwords	*/
#define SM_TIMED_EX (1L<<4) 	/* Timed event must run exclusively 		*/
#define SM_TIMED_SH (1L<<5) 	/* Shrink to run timed event				*/
#define SM_ANON_EM	(1L<<6) 	/* Allow anonymous e-mail					*/
#define SM_LISTLOC	(1L<<7) 	/* Use location of caller in user lists 	*/
#define SM_WILDCAT	(1L<<8) 	/* Expand Wildcat color codes in messages	*/
#define SM_PCBOARD	(1L<<9) 	/* Expand PCBoard color codes in messages	*/
#define SM_WWIV 	(1L<<10)	/* Expand WWIV color codes in messages		*/
#define SM_CELERITY (1L<<11)	/* Expand Celerity color codes in messages	*/
#define SM_RENEGADE (1L<<12)	/* Expand Renegade color codes in messages */
#define SM_ECHO_PW	(1L<<13)	/* Echo passwords locally					*/
#define SM_REQ_PW	(1L<<14)	/* Require passwords locally				*/
#define SM_L_SYSOP	(1L<<15)	/* Allow local sysop logon/commands 		*/
#define SM_R_SYSOP	(1L<<16)	/* Allow remote sysop logon/commands		*/
#define SM_QUOTE_EM (1L<<17)	/* Allow quoting of e-mail					*/
#define SM_EURODATE (1L<<18)	/* Europian date format (DD/MM/YY)			*/
#define SM_MILITARY (1L<<19)	/* Military time format 					*/
#define SM_TIMEBANK (1L<<20)	/* Allow time bank functions				*/
#define SM_FILE_EM	(1L<<21)	/* Allow file attahcments in E-mail 		*/
#define SM_SHRTPAGE (1L<<22)	/* Short sysop page 						*/

							/* Different bits in node_misc				*/
#define NM_ANSALARM (1<<0)	/* Alarm locally on answer					*/
#define NM_WFCSCRN  (1<<1)	/* Wait for call screen                     */
#define NM_MERGETXT (1<<2)	/* Merge duplicate strings from TEXT.DAT	*/
#define NM_LCL_EDIT (1<<3)	/* Use local editor to create messages		*/
#define NM_EMSOVL	(1<<4)	/* Use expanded memory of overlays			*/
#define NM_WINOS2	(1<<5)	/* Use Windows/OS2 time slice API call		*/
#define NM_INT28	(1<<6)	/* Make int 28 DOS idle calls				*/
#define NM_NODV 	(1<<7)	/* Don't detect and use DESQview API        */

							/* Miscellaneous Modem Settings (mdm_misc)  */
#define MDM_CTS 	 (1<<0) /* Use hardware send flow control			*/
#define MDM_RTS 	 (1<<1) /* Use hardware recv flow control			*/
#define MDM_STAYHIGH (1<<2) /* Stay at highest DTE rate 				*/
#define MDM_CALLERID (1<<3) /* Supports Caller ID						*/
#define MDM_DUMB	 (1<<4) /* Just watch DCD for answer - dumb modem	*/

									/* Bit values for prot[x].misc */
#define PROT_DSZLOG (1<<0)          /* Supports DSZ Log */

									/* Bit values in netmail_misc */

#define NMAIL_ALLOW (1<<0)			/* Allow NetMail */
#define NMAIL_CRASH (1<<1)			/* Default netmail to crash */
#define NMAIL_HOLD	(1<<2)			/* Default netmail to hold */
#define NMAIL_KILL	(1<<3)			/* Default netmail to kill after sent */
#define NMAIL_ALIAS (1<<4)			/* Use Aliases in NetMail */

									/* Attribute bits for fido msg header */
#define FIDO_PRIVATE	(1<<0)		/* Private message */
#define FIDO_CRASH		(1<<1)		/* Crash-mail (send immediately) */
#define FIDO_RECV		(1<<2)		/* Received successfully */
#define FIDO_SENT		(1<<3)		/* Sent successfully */
#define FIDO_FILE		(1<<4)		/* File attached */
#define FIDO_INTRANS	(1<<5)		/* In transit */
#define FIDO_ORPHAN 	(1<<6)		/* Orphan */
#define FIDO_KILLSENT	(1<<7)		/* Kill it after sending it */
#define FIDO_LOCAL		(1<<8)		/* Created locally - on this system */
#define FIDO_HOLD		(1<<9)		/* Hold - don't send it yet */
#define FIDO_FREQ		(1<<11) 	/* File request */
#define FIDO_RRREQ		(1<<12) 	/* Return receipt request */
#define FIDO_RR 		(1<<13) 	/* This is a return receipt */
#define FIDO_AUDIT		(1<<14) 	/* Audit request */
#define FIDO_FUPREQ 	(1<<15) 	/* File update request */


									/* Bit values for sub[x].misc */
#define SUB_NSCAN	(1L<<0) 		/* Scan this sub-board for new msgs */
#define SUB_SSCAN	(1L<<1) 		/* Scan this sub-board for msgs to you */
#define SUB_QNET	(1L<<2) 		/* Sub-board is netted via QWK network */
#define SUB_PNET	(1L<<3) 		/* Sub-board is netted via PostLink */
#define SUB_FIDO	(1L<<4) 		/* Sub-board is netted via FidoNet */
#define SUB_PRIV	(1L<<5) 		/* Allow private posts on sub */
#define SUB_PONLY	(1L<<6) 		/* Private posts only */
#define SUB_ANON	(1L<<7) 		/* Allow anonymous posts on sub */
#define SUB_AONLY	(1L<<8) 		/* Anonymous only */
#define SUB_NAME	(1L<<9) 		/* Must use real names */
#define SUB_DEL 	(1L<<10)		/* Allow users to delete messages */
#define SUB_DELLAST (1L<<11)		/* Allow users to delete last msg only */
#define SUB_FORCED	(1L<<12)		/* Sub-board is forced scanning */
#define SUB_NOTAG	(1L<<13)		/* Don't add tag or origin lines */
#define SUB_TOUSER	(1L<<14)		/* Prompt for to user on posts */
#define SUB_ASCII	(1L<<15)		/* ASCII characters only */
#define SUB_QUOTE	(1L<<16)		/* Allow quoting */
#define SUB_NSDEF	(1L<<17)		/* New-Scan on by default */

                                    /* Bit values for dir[x].misc */
#define DIR_FCHK    (1<<0)          /* Check for file existance */
#define DIR_RATE    (1<<1)          /* Force uploads to be rated G,R, or X */
#define DIR_MULT    (1<<2)          /* Ask for multi-disk numbering */
#define DIR_EXTDESC (1<<3)          /* Extended descriptions allowed */
#define DIR_FREE	(1<<4)			/* Free downloads */
#define DIR_TFREE	(1<<5)			/* Time to download is free */
#define DIR_CDTUL	(1<<6)			/* Credit Uploads */
#define DIR_CDTDL	(1<<7)			/* Credit Downloads */
#define DIR_ANON	(1<<8)			/* Anonymous uploads */
#define DIR_AONLY	(1<<9)			/* Anonymous only */
#define DIR_ULDATE	(1<<10) 		/* Include upload date in listing */
#define DIR_DIZ 	(1<<11) 		/* FILE_ID.DIZ and DESC.SDI support */

                                    /* Bit values for file_t.misc */
#define FM_EXTDESC  (1<<0)          /* Extended description exists */
#define FM_ANON 	(1<<1)			/* Anonymous upload */

enum {								/* errormsg() codes */
	 ERR_OPEN						/* opening a file */
	,ERR_CLOSE						/* close a file */
	,ERR_FDOPEN 					/* associating a stream with fd */
	,ERR_READ						/* reading from file */
	,ERR_WRITE						/* writing to file */
	,ERR_REMOVE 					/* removing a file */
	,ERR_ALLOC						/* allocating memory */
	,ERR_CHK						/* checking */
	,ERR_LEN						/* file length */
	,ERR_EXEC						/* executing */
	,ERR_CHDIR						/* changing directory */
	};

enum {								/* Access requirement binaries */
     AR_NULL
    ,AR_OR
    ,AR_NOT
    ,AR_EQUAL
    ,AR_BEGNEST
    ,AR_ENDNEST
    ,AR_LEVEL
    ,AR_AGE
    ,AR_BPS
    ,AR_NODE
    ,AR_TLEFT
    ,AR_TUSED
    ,AR_USER
	,AR_TIME
    ,AR_PCR
	,AR_FLAG1
	,AR_FLAG2
	,AR_FLAG3
	,AR_FLAG4
	,AR_EXEMPT
	,AR_REST
    ,AR_SEX
	,AR_UDR
	,AR_UDFR
	,AR_EXPIRE
	,AR_CREDIT
	,AR_DAY
    };

#define SBBS_MAIN_CMDS "ABCDEFGINOPQRSTUXZ&*JM"

enum {								/* Main menu commands */
	 MAIN_CMD_AUTOMSG				/* Auto-message */
	,MAIN_CMD_BROWSE				/* Message browse */
	,MAIN_CMD_CHAT					/* Chat section */
	,MAIN_CMD_DEFAULTS				/* Default configuration */
	,MAIN_CMD_EMAIL 				/* E-mail menu */
	,MAIN_CMD_FINDTEXT				/* Find text in messages */
	,MAIN_CMD_GFILES				/* General text file sections */
	,MAIN_CMD_INFO					/* Information */
	,MAIN_CMD_NEWSCAN				/* New message scan */
	,MAIN_CMD_OFFLINE				/* Offline */
	,MAIN_CMD_POST					/* Post message */
	,MAIN_CMD_QWK					/* QWK */
	,MAIN_CMD_READMSGS				/* Read Messages */
	,MAIN_CMD_SEARCH				/* Search for messages to you */
	,MAIN_CMD_TRANSFER				/* Transfer section */
	,MAIN_CMD_USERLIST				/* User lists */
	,MAIN_CMD_XTRN					/* External programs */
	,MAIN_CMD_ZNEWSCAN				/* Continuous new message scan */
	,MAIN_CMD_CONFIG				/* Configuration commands */
	,MAIN_CMD_AREAS 				/* View areas */
	,MAIN_CMD_JUMP					/* Jump message area */
	,MAIN_CMD_MINBANK				/* Minute bank */

	,MAIN_CMDS
	};

#define SBBS_XFER_CMDS "BCDEFILNOQRSTUVZ&*J"

enum {								/* Transfer section commands */
	 XFER_CMD_BATCH 				/* Batch/Bimodem transfer commands */
	,XFER_CMD_CHAT					/* Chat section */
	,XFER_CMD_DOWNLOAD				/* Download */
	,XFER_CMD_EXTINFO				/* Extended file information */
	,XFER_CMD_FINDTEXT				/* Find text in desccriptions */
	,XFER_CMD_INFO					/* Information */
	,XFER_CMD_LISTFILES 			/* List files */
	,XFER_CMD_NEWSCAN				/* New file scan */
	,XFER_CMD_OFFLINE				/* Offline */
	,XFER_CMD_QUIT					/* Quit to main menu */
	,XFER_CMD_REMOVE_EDIT			/* Remove/Edit a file */
	,XFER_CMD_SEARCH				/* Search for filename */
	,XFER_CMD_TEMP					/* Temp directory commands */
	,XFER_CMD_UPLOAD				/* Upload */
	,XFER_CMD_VIEW					/* View files */
	,XFER_CMD_SYSOP_UPLOAD			/* Upload file to sysop */
	,XFER_CMD_CONFIG				/* Configuration commands */
	,XFER_CMD_AREAS 				/* Area change */
	,XFER_CMD_JUMP					/* Jump transfer area */

	,XFER_CMDS
	};

#define SBBS_SCAN_CMDS "ABCDFILMPQRTY-"

enum {                              /* Message scanning commands */
     SCAN_CMD_AUTOREPLY             /* Auto-reply */
    ,SCAN_CMD_BYPASS                /* Bypass sub-board */
	,SCAN_CMD_CONTINUOUS			/* Continuous read */
    ,SCAN_CMD_DELETE                /* Delete */
    ,SCAN_CMD_FINDTEXT              /* Find text */
    ,SCAN_CMD_INFO                  /* Information on sub */
    ,SCAN_CMD_LIST                  /* List messages */
    ,SCAN_CMD_MAILREPLY             /* Reply in E-mail */
    ,SCAN_CMD_POST                  /* Post */
    ,SCAN_CMD_QUIT                  /* Quit */
    ,SCAN_CMD_REREAD                /* Reread */
    ,SCAN_CMD_TENTITLES             /* Next ten titles */
	,SCAN_CMD_TOYOU 				/* Messages to you */
    ,SCAN_CMD_PREVIOUS              /* Previous message */

    ,SCAN_CMDS
    };

#define SBBS_MAIL_CMDS "ADFLQR-"

enum {								/* E-mail reading commands */
	 MAIL_CMD_AUTOREPLY 			/* Auto-reply */
	,MAIL_CMD_DELETE				/* Delete last */
	,MAIL_CMD_FORWARD				/* Forward last */
	,MAIL_CMD_LIST					/* List titles */
	,MAIL_CMD_QUIT					/* Quit to main menu */
	,MAIL_CMD_REREAD				/* Reread mail */
	,MAIL_CMD_PREVIOUS				/* Previous mail */

	,MAIL_CMDS
	};

enum {                              /* Values for dir[x].sort */
     SORT_NAME_A                    /* Sort by filename, ascending */
    ,SORT_NAME_D                    /* Sort by filename, descending */
    ,SORT_DATE_A                    /* Sort by upload date, ascending */
    ,SORT_DATE_D                    /* Sort by upload date, descending */
    };

enum {
	 clr_mnehigh
	,clr_mnelow
	,clr_mnecmd
	,clr_inputline
	,clr_err
	,clr_nodenum
	,clr_nodeuser
	,clr_nodestatus
	,clr_filename
	,clr_filecdt
	,clr_filedesc
	,clr_filelsthdrbox
	,clr_filelstline
	,clr_chatlocal
	,clr_chatremote
	,clr_multichat
	,TOTAL_COLORS };

enum {								/* Values for xtrn_t.type			*/
	 XTRN_NONE						/* No data file needed				*/
	,XTRN_SBBS						/* Synchronet external				*/
	,XTRN_WWIV						/* WWIV external					*/
	,XTRN_GAP						/* Gap door 						*/
	,XTRN_RBBS						/* RBBS, QBBS, or Remote Access 	*/
	,XTRN_WILDCAT					/* Wildcat							*/
	,XTRN_PCBOARD					/* PCBoard							*/
	,XTRN_SPITFIRE					/* SpitFire 						*/
	,XTRN_UTI						/* UTI Doors - MegaMail 			*/
	,XTRN_SR						/* Solar Realms 					*/
	};

enum {								/* Values for xtrn_t.event			*/
	 EVENT_NONE 					/* Only accessible by menu			*/
	,EVENT_LOGON					/* Execute during logon sequence	*/
	,EVENT_LOGOFF					/* Execute during logoff sequence	*/
	,EVENT_NEWUSER					/* Execute during newuser app.		*/
	,EVENT_BIRTHDAY 				/* Execute on birthday				*/
	};

									/* Mode bits for QWK stuff */
#define A_EXPAND	(1<<0)			/* Expand to ANSI sequences */
#define A_LEAVE 	(1<<1)			/* Leave in */
#define A_STRIP 	(1<<2)			/* Strip out */

									/* Bits in xtrn_t.misc				*/
#define MULTIUSER	(1L<<0) 		/* allow multi simultaneous users	*/
#define ANSI		(1L<<1) 		/* user must have ANSI, same as ^^^ */
#define IO_INTS 	(1L<<2) 		/* Intercept I/O interrupts 		*/
#define SHRINK		(1L<<3) 		/* Shrink to run external program	*/
#define MODUSERDAT	(1L<<4) 		/* Program can modify user data 	*/
#define WWIVCOLOR	(1L<<5) 		/* Program uses WWIV color codes	*/
#define EVENTONLY	(1L<<6) 		/* Program executes as event only	*/
#define STARTUPDIR	(1L<<7) 		/* Create drop file in start-up dir */
#define REALNAME	(1L<<8) 		/* Use real name in drop file		*/

#define INVALID_DIR 0xffff          /* Invalid directory value          */

#define KEY_BUFSIZE 1024	/* Size of keyboard input buffer			*/
#define SAVE_LINES	 4		/* Maximum number of lines to save			*/
#define LINE_BUFSIZE 512	/* Size of line output buffer               */

							/* Various Character/Ctrl-Code Definitions	*/
#define SP			32		/* Space bar								*/
#define ESC			27		/* ESC Char 								*/
#define	CR			13		/* Carriage Return							*/
#define FF          12      /* Form Feed								*/
#define	LF			10		/* Line Feed								*/
#define TAB			9		/* Horizontal Tabulation					*/
#define BS			8		/* Back Space								*/
#define STX         2		/* Start of text                            */
#define ETX         3		/* End of text	                            */

#define TABSIZE		4		/* Tab Size									*/

#define DSTSDABLEN	50		/* Length of DSTS.DAB file					*/

							/* Console I/O Bits	(console)				*/
#define CON_R_ECHO	 (1<<0)	/* Echo remotely							*/
#define CON_R_ECHOX	 (1<<1)	/* Echo X's to remote user					*/
#define CON_R_INPUT  (1<<2)	/* Accept input remotely					*/
#define CON_L_ECHO	 (1<<3)	/* Echo locally              				*/
#define CON_L_ECHOX	 (1<<4) /* Echo X's locally							*/
#define CON_L_INPUT  (1<<5)	/* Accept input locally						*/
#define CON_R0DENTI  (1<<6) /* r0dent input is active                   */
#define CON_R0DENTO  (1<<7) /* r0dent output is active                  */
#define CON_RAW_IN   (1<<8) /* Raw input mode - no editing capabilities */
#define CON_ELITE	 (1<<9)	/* Elite text mode is active				*/
#define CON_ECHO_OFF (1<<10)/* Remote & Local echo disabled for ML/MF	*/
#define CON_UPARROW  (1<<11)/* Up arrow hit - move up one line			*/

							/* Number of milliseconds                   */
#define DELAY_HANGUP 250    /* Delay before modem drops carrier         */
#define DELAY_MDMTLD 500    /* Delay to give each ~ in modem strings    */
#define DELAY_SPIN   10     /* Delay for the spinning cursor            */
#define DELAY_AUTOHG 1500	/* Delay for auto-hangup (xfer) 			*/

#define SEC_LOGON	1800	/* 30 minutes allowed to logon				*/
#define SEC_WARN     180    /* Warn inactive user after 3 minutes       */
#define SEC_HANGUP   300    /* Hangup on an inactive user after 5 mins  */
#define SEC_BILLING   90	/* under 2 minutes per billing call 		*/
#define SEC_OK		   5	/* Attempt to get an OK response from modem */
#define SEC_ANSI	   5	/* Attempt to get a valid ANSI response 	*/
#define SEC_ANSWER	  30	/* Retries to get an answer code from modem */
#define SEC_CID 	  10	/* Ten second pause for caller ID			*/

#define LOOP_NOPEN	 500	/* Retries before file access denied		*/
#define LOOP_NODEDAB 100	/* Retries on NODE.DAB locking/unlocking	*/

							/* String lengths							*/
#define LEN_ALIAS		25	/* User alias								*/
#define LEN_NAME		25	/* User name								*/
#define LEN_HANDLE		8	/* User chat handle 						*/
#define LEN_NOTE		30	/* User note								*/
#define LEN_COMP		30	/* User computer description				*/
#define LEN_COMMENT 	60	/* User comment 							*/
#define LEN_NETMAIL 	60	/* NetMail forwarding address				*/
#define LEN_PASS		 8	/* User password							*/
#define LEN_PHONE		12	/* User phone number						*/
#define LEN_BIRTH		 8	/* Birthday in MM/DD/YY format				*/
#define LEN_ADDRESS 	30	/* User address 							*/
#define LEN_LOCATION	30	/* Location (City, State)					*/
#define LEN_ZIPCODE 	10	/* Zip/Postal code							*/
#define LEN_MODEM		 8	/* User modem type description				*/
#define LEN_FDESC		58	/* File description 						*/
#define LEN_TITLE		70	/* Message title							*/
#define LEN_MAIN_CMD	40	/* Storage in user.dat for custom commands	*/
#define LEN_XFER_CMD	40
#define LEN_SCAN_CMD	40
#define LEN_MAIL_CMD	40
#define LEN_CID 		25	/* Caller ID (phone number) 				*/
#define LEN_ARSTR		40	/* Max length of Access Requirement string	*/
#define LEN_CHATACTCMD	 9	/* Chat action command						*/
#define LEN_CHATACTOUT	65	/* Chat action output string				*/

/****************************************************************************/
/* This is a list of offsets into the USER.DAT file for different variables */
/* that are stored (for each user)											*/
/****************************************************************************/
#define U_ALIAS 	0					/* Offset to alias */
#define U_NAME		(U_ALIAS+LEN_ALIAS) /* Offset to name */
#define U_HANDLE	(U_NAME+LEN_NAME)
#define U_NOTE		(U_HANDLE+LEN_HANDLE+2)
#define U_COMP		(U_NOTE+LEN_NOTE)
#define U_COMMENT	(U_COMP+LEN_COMP+2)

#define U_NETMAIL	(U_COMMENT+LEN_COMMENT+2)

#define U_ADDRESS	(U_NETMAIL+LEN_NETMAIL+2)
#define U_LOCATION	(U_ADDRESS+LEN_ADDRESS)
#define U_ZIPCODE	(U_LOCATION+LEN_LOCATION)

#define U_PASS		(U_ZIPCODE+LEN_ZIPCODE+2)
#define U_PHONE  	(U_PASS+8) 			/* Offset to phone-number */
#define U_BIRTH  	(U_PHONE+12)		/* Offset to users birthday	*/
#define U_MODEM     (U_BIRTH+8)
#define U_LASTON	(U_MODEM+8)
#define U_FIRSTON	(U_LASTON+8)
#define U_EXPIRE    (U_FIRSTON+8)
#define U_PWMOD     (U_EXPIRE+8)

#define U_LOGONS    (U_PWMOD+8+2)
#define U_LTODAY    (U_LOGONS+5)
#define U_TIMEON    (U_LTODAY+5)
#define U_TEXTRA  	(U_TIMEON+5)
#define U_TTODAY    (U_TEXTRA+5)
#define U_TLAST     (U_TTODAY+5)
#define U_POSTS     (U_TLAST+5)
#define U_EMAILS    (U_POSTS+5)
#define U_FBACKS    (U_EMAILS+5)
#define U_ETODAY	(U_FBACKS+5)
#define U_PTODAY	(U_ETODAY+5)

#define U_ULB       (U_PTODAY+5+2)
#define U_ULS       (U_ULB+10)
#define U_DLB       (U_ULS+5)
#define U_DLS       (U_DLB+10)
#define U_CDT		(U_DLS+5)
#define U_MIN		(U_CDT+10)

#define U_LEVEL 	(U_MIN+10+2)	/* Offset to Security Level    */
#define U_FLAGS1	(U_LEVEL+2) 	/* Offset to Flags */
#define U_TL		(U_FLAGS1+8)	/* Offset to unused field */
#define U_FLAGS2	(U_TL+2)		/* Offset to unused field */
#define U_EXEMPT	(U_FLAGS2+8)
#define U_REST		(U_EXEMPT+8)
#define U_ROWS		(U_REST+8+2)	/* Number of Rows on user's monitor */
#define U_SEX		(U_ROWS+2)		/* Sex, Del, ANSI, color etc.		*/
#define U_MISC		(U_SEX+1)		/* Miscellaneous flags in 8byte hex */
#define U_XEDIT 	(U_MISC+8)		/* External editor */
#define U_LEECH 	(U_XEDIT+2) 	/* two hex digits - leech attempt count */
#define U_CURGRP	(U_LEECH+2) 	/* Current group */
#define U_CURSUB	(U_CURGRP+4)	/* Current sub-board */
#define U_CURLIB	(U_CURSUB+4)	/* Current library */
#define U_CURDIR	(U_CURLIB+4)	/* Current directory */
#define U_CMDSET	(U_CURDIR+4)	/* User's command set */
#define U_MAIN_CMD	(U_CMDSET+2+2)	/* Custom main command set */
#define U_XFER_CMD	(U_MAIN_CMD+LEN_MAIN_CMD)
#define U_SCAN_CMD	(U_XFER_CMD+LEN_XFER_CMD+2)
#define U_MAIL_CMD	(U_SCAN_CMD+LEN_SCAN_CMD)
#define U_FREECDT	(U_MAIL_CMD+LEN_MAIL_CMD+2)  /* Unused bytes */
#define U_FLAGS3	(U_FREECDT+10)	/* Unused bytes */
#define U_FLAGS4	(U_FLAGS3+8)
#define U_UNUSED1	(U_FLAGS4+8)
#define U_UNUSED2	(U_UNUSED1+22+2)
#define U_LEN		(U_UNUSED2+48+2)

/****************************************************************************/
/* Offsets into DIR .DAT file for different fields for each file 			*/
/****************************************************************************/
#define F_CDT		0				/* Offset in DIR#.DAT file for cdts */
#define F_DESC		(F_CDT+7+2) 	/* Description						*/
#define F_ULER		(F_DESC+58+2)	/* Uploader							*/
#define F_TIMESDLED (F_ULER+30+2) 	/* Number of times downloaded 		*/
#define F_OPENCOUNT	(F_TIMESDLED+5+2)
#define F_MISC		(F_OPENCOUNT+3+2)
#define F_ALTPATH	(F_MISC+1)		/* Two hex digit alternate path */
#define F_LEN		(F_ALTPATH+2+2) /* Total length of all fdat in file */

#define F_IXBSIZE	22				/* Length of each index entry		*/

enum {								/* Node Status */
	 NODE_WFC			        	/* Waiting for Call */
	,NODE_LOGON                  	/* at logon prompt */
	,NODE_NEWUSER         			/* New user applying */
	,NODE_INUSE			 			/* In Use */
	,NODE_QUIET			 			/* In Use - quiet mode */
	,NODE_OFFLINE		 			/* Offline */
	,NODE_NETTING		 			/* Networking */
	,NODE_EVENT_WAITING				/* Waiting for all nodes to be inactive */
	,NODE_EVENT_RUNNING				/* Running an external event */
	,NODE_EVENT_LIMBO				/* Allowing another node to run an event */
	};

									/* Bit values for node.misc */
#define	NODE_ANON	(1<<0)			/* Anonymous User */
#define	NODE_LOCK	(1<<1)			/* Locked for sysops only */
#define	NODE_INTR	(1<<2)			/* Interrupted - hang up */
#define NODE_MSGW	(1<<3)			/* Message is waiting (old way) */
#define NODE_POFF   (1<<4)        	/* Page disabled */
#define NODE_AOFF	(1<<5)			/* Activity Alert disabled */
#define NODE_UDAT	(1<<6)			/* User data has been updated */
#define NODE_RRUN	(1<<7)			/* Re-run this node when log off */
#define NODE_EVENT	(1<<8)			/* Must run node event after log off */
#define NODE_DOWN	(1<<9)			/* Down this node after logoff */
#define NODE_RPCHT	(1<<10) 		/* Reset private chat */
#define NODE_NMSG	(1<<11) 		/* Node message waiting (new way) */

enum {								/* Node Action */
	 NODE_MAIN						/* Main Prompt */
	,NODE_RMSG			 			/* Reading Messages */
	,NODE_RMAL						/* Reading Mail */
	,NODE_SMAL			 			/* Sending Mail */
	,NODE_RTXT			 			/* Reading G-Files */
	,NODE_RSML			 			/* Reading Sent Mail */
	,NODE_PMSG			 			/* Posting Message */
	,NODE_AMSG			 			/* Auto-message */
	,NODE_XTRN			  			/* Running External Program */
	,NODE_DFLT			  			/* Main Defaults Section */
	,NODE_XFER			   			/* Transfer Prompt */
	,NODE_DLNG		    			/* Downloading File */
	,NODE_ULNG			    		/* Uploading File */
	,NODE_BXFR						/* Bidirectional Transfer */
	,NODE_LFIL			    		/* Listing Files */
	,NODE_LOGN			    		/* Logging on */
	,NODE_LCHT			    		/* In Local Chat with Sysop */
	,NODE_MCHT		     			/* In Multi-Chat with Other Nodes */
	,NODE_GCHT			   			/* In Local Chat with Guru */
	,NODE_CHAT			   			/* In Chat Section */
	,NODE_SYSP		     			/* Sysop Activity */
	,NODE_TQWK						/* Transferring QWK packet */
	,NODE_PCHT						/* In Private Chat */
	,NODE_PAGE						/* Paging another node for Private Chat */
	,NODE_RFSD						/* Retrieving file from seq dev (aux=dev)*/
	};

#define SIF_MAXBUF  0x7000			/* Maximum buffer size of SIF data */

/* NOTE: Do not change the values of the following block of defines!	*/

#define DELETED 	(1L<<0) 		/* Bit values for user.misc 		*/
#define ANSI		(1L<<1) 		/* Supports ANSI terminal emulation */
#define COLOR		(1L<<2) 		/* Send color codes 				*/
#define RIP 		(1L<<3) 		/* Supports RIP terminal emulation	*/
#define UPAUSE		(1L<<4) 		/* Pause on every screen full		*/
#define SPIN		(1L<<5) 		/* Spinning cursor - Same as K_SPIN */
#define INACTIVE	(1L<<6) 		/* Inactive user slot				*/
#define EXPERT		(1L<<7) 		/* Expert menu mode 				*/
#define ANFSCAN 	(1L<<8) 		/* Auto New file scan				*/
#define CLRSCRN 	(1L<<9) 		/* Clear screen before each message */
#define QUIET		(1L<<10)		/* Quiet mode upon logon			*/
#define BATCHFLAG	(1L<<11)		/* File list allow batch dl flags	*/
#define NETMAIL 	(1L<<12)		/* Forward e-mail to fidonet addr	*/
#define CURSUB		(1L<<13)		/* Remember current sub-board/dir	*/
#define ASK_NSCAN	(1L<<14)		/* Ask for newscanning upon logon	*/
#define NO_EXASCII	(1L<<15)		/* Don't send extended ASCII        */
#define ASK_SSCAN	(1L<<16)		/* Ask for messages to you at logon */
#define AUTOTERM	(1L<<17)		/* Autodetect terminal type 		*/

#define CLREOL      256     /* Character to erase to end of line 		*/
#define HIGH        8       /* High intensity for curatr             */

							/* Online status (online)					*/
#define ON_LOCAL	1	 	/* Online locally							*/
#define ON_REMOTE   2  		/* Online remotely							*/
#define ON_XFER		3		/* Online remotely - transferring file		*/
#define ON_WFC		4		/* Online waiting for a call				*/

							/* Varios SYSTEM parameters for sys_status	*/
#define SS_LOGOPEN	(1L<<0)	/* Node's Log file is open          		*/
#define SS_INITIAL  (1L<<1)	/* The bbs data has been initialized.       */
#define SS_TMPSYSOP (1L<<2)	/* Temporary Sysop Status					*/
#define SS_USERON   (1L<<3)	/* A User is logged on to the BBS			*/
#define SS_LCHAT    (1L<<4) /* Local chat in progress					*/
#define SS_CAP		(1L<<5)	/* Capture is on							*/
#define SS_ANSCAP	(1L<<6) /* Capture ANSI codes too					*/
#define SS_FINPUT	(1L<<7) /* Using file for input 					*/
#define SS_COMISR	(1L<<8) /* Com port ISR is installed				*/
#define SS_DAILY	(1L<<9) /* Execute System Daily Event on logoff 	*/
#define SS_INUEDIT	(1L<<10) /* Inside Alt-Useredit section 			*/
#define SS_ABORT	(1L<<11) /* Global abort input or output flag		*/
#define SS_SYSPAGE	(1L<<12) /* Paging sysop							*/
#define SS_SYSALERT (1L<<13) /* Notify sysop when users hangs up		*/
#define SS_GURUCHAT (1L<<14) /* Guru chat in progress					*/
#define SS_RETURN	(1L<<15) /* Board is returning from an exec call	*/
#define SS_NODEDAB	(1L<<16) /* NODE.DAB operations are okay			*/
#define SS_EVENT	(1L<<17) /* Time shortened due to upcoming event	*/
#define SS_PAUSEON	(1L<<18) /* Pause on, overriding user default		*/
#define SS_PAUSEOFF (1L<<19) /* Pause off, overriding user default		*/
#define SS_IN_CTRLP (1L<<20) /* Inside ctrl-p send node message func	*/
#define SS_NEWUSER	(1L<<21) /* New User online 						*/
#define SS_MDMDEBUG (1L<<22) /* Modem debug output						*/
#define SS_NEST_PF	(1L<<23) /* Nested in printfile function			*/
#define SS_DCDHIGH	(1L<<24) /* Assume DCD is high always				*/

enum {						/* Different values for the current section */
	 SEC_MAIN				/* Main section								*/
	,SEC_XFER				/* Xfer section								*/
	,SEC_XTRN				/* External program section 				*/
	};

							/* Bits in 'mode' for getkey and getstr		*/
#define K_UPPER		(1<<0)	/* Converts all letters to upper case 		*/
#define K_UPRLWR    (1<<1)  /* Upper/Lower case automatically           */
#define K_NUMBER    (1<<2)  /* Allow numbers only                       */
#define K_WRAP	    (1<<3)  /* Allows word wrap							*/
#define K_MSG		(1<<4)  /* Allows ANSI, ^N ^A ^G					*/
#define K_SPIN      (1<<5)  /* Spinning cursor (same as SPIN)           */
#define K_LINE      (1<<6)  /* Input line (inverse color)               */
#define K_EDIT      (1<<7)  /* Edit string passed                       */
#define K_CHAT		(1<<8)	/* In chat multi-chat 						*/
#define K_NOCRLF    (1<<9)	/* Don't print CRLF after string input      */
#define K_ALPHA 	(1<<10)	/* Only allow alphabetic characters			*/
#define K_GETSTR	(1<<11) /* getkey called from getstr()				*/
#define K_LOWPRIO	(1<<12) /* low priority input						*/
#define K_NOEXASC	(1<<13) /* No extended ASCII allowed				*/
#define K_E71DETECT (1<<14) /* Detect E-7-1 terminal type				*/
#define K_AUTODEL	(1<<15) /* Auto-delete text (used with K_EDIT)		*/

							/* Bits in 'mode' for putmsg and printfile  */
#define P_NOABORT  	(1<<0)  /* Disallows abortion of a message          */
#define P_SAVEATR   (1<<1)  /* Save the new current attributres after	*/
							/* msg has printed. */
#define P_NOATCODES (1<<2)	/* Don't allow @ codes                      */

							/* Bits in 'mode' for listfiles             */
#define FL_ULTIME   (1<<0)  /* List files by upload time                */
#define FL_DLTIME   (1<<1)  /* List files by download time              */
#define FL_NO_HDR   (1<<2)  /* Don't list directory header              */
#define FL_FINDDESC (1<<3)  /* Find text in description                 */
#define FL_EXFIND   (1<<4)	/* Find text in description - extended info */
#define FL_VIEW     (1<<5)	/* View ZIP/ARC/GIF etc. info               */

							/* Bits in the mode of writemsg and email() */
#define WM_EXTDESC	(1<<0)	/* Writing extended file description		*/
#define WM_EMAIL	(1<<1)	/* Writing e-mail							*/
#define WM_NETMAIL	(1<<2)	/* Writing NetMail							*/
#define WM_ANON 	(1<<3)	/* Writing anonymous message				*/
#define WM_FILE 	(1<<4)	/* Attaching a file to the message			*/

							/* Message mode bits						*/
#define MSG_PERM	1		/* Permanent - non-purgable message (post)  */
#define MSG_FORWARD 1       /* Forwarded message (mail)                 */
#define MSG_ANON	2		/* Anonymous message						*/
#define MSG_PRIVATE 4		/* Private posted message					*/
#define MSG_READ	8		/* Private post has been read				*/
#define MSG_FILE	16		/* File attached							*/

							/* Bits in the mode of external()           */
#define EX_CC       (1<<0)	/* Use command.com to load other process    */
#define EX_OUTR     (1<<1)  /* Copy DOS output to remote                */
#define EX_OUTL 	(1<<2)	/* Use _lputc() for local DOS output		*/
#define EX_INR		(1<<3)	/* Trap int 16h keyboard input requests     */
#define EX_WWIV 	(1<<4)	/* Expand WWIV color codes to ANSI sequence */

enum {						/* Values for 'mode' in listfileinfo        */
	 FI_INFO            	/* Just list file information               */
	,FI_REMOVE           	/* Remove/Move/Edit file information        */
	,FI_DOWNLOAD         	/* Download files                           */
	,FI_OLD              	/* Search/Remove files not downloaded since */
	,FI_OLDUL	 			/* Search/Remove files uploaded before      */
	,FI_OFFLINE   			/* Search/Remove files not online			*/
	,FI_USERXFER  			/* User Xfer Download                       */
	,FI_CLOSE 	  			/* Close any open records					*/
	};

#define L_LOGON     1       /* Logon List maintenance                   */
#define LOL_SIZE    81      /* Length of each logon list entry          */

#define SUB_CRCS	2000	/* Total number of CRCs stored for per sub	*/

							/* Bits in mode of scanposts() function 	*/
#define SCAN_CONST	(1<<0)	/* Continuous message scanning				*/
#define SCAN_NEW	(1<<1)	/* New scanning								*/
#define SCAN_BACK	(1<<2)	/* Scan the last message if no new			*/
#define SCAN_TOYOU	(1<<3)	/* Scan for messages to you 				*/
#define SCAN_FIND	(1<<4)	/* Scan for text in messages				*/

							/* Bits in misc of chan_t					*/
#define CHAN_PW 	(1<<0)	/* Can be password protected				*/
#define CHAN_GURU	(1<<1)	/* Guru joins empty channel 				*/

enum {						/* Values of mode for userlist function     */
	 UL_ALL					/* List all users in userlist				*/
	,UL_SUB      			/* List all users with access to cursub     */
	,UL_DIR					/* List all users with access to curdir 	*/
	};


#define BO_LEN		16		/* BACKOUT.DAB record length				*/

#define BO_OPENFILE 0		/* Backout types */

/************************/
/* Remote I/O Constants */
/************************/

							/* i/o mode and state flags */
#define CTSCK 0x1000     	/* check cts (mode only) */
#define RTSCK 0x2000		/* check rts (mode only) */
#define TXBOF 0x0800		/* transmit buffer overflow (outcom only) */
#define ABORT 0x0400     	/* check for ^C (mode), aborting (state) */
#define PAUSE 0x0200     	/* check for ^S (mode), pausing (state) */
#define NOINP 0x0100     	/* input buffer empty (incom only) */

							/* status flags */
#define DCD 	0x80       	/* DCD on */
#define RI		0x40		/* Ring indicate */
#define DSR 	0x20		/* Dataset ready */
#define CTS 	0x10       	/* CTS on */
#define FERR 	0x08		/* Frameing error */
#define PERR 	0x04		/* Parity error */
#define OVRR 	0x02		/* Overrun */
#define RXLOST 	0x01       	/* Receive buffer overflow */

/* rioctl() arguments */
/* returns mode or state flags in high 8 bits, status flags in low 8 bits */

							/* the following return mode in high 8 bits */
#define IOMODE 0        	/* no operation */
#define IOSM 1          	/* i/o set mode flags */
#define IOCM 2          	/* i/o clear mode flags */

#define GVERS 0x007 		/* get version */
#define GUART 0x107 		/* get uart */
#define GIRQN 0x207 		/* get IRQ number */
#define GBAUD 0x307 		/* get baud */

							/* the following return state in high 8 bits */
#define IOSTATE 4       	/* no operation */
#define IOSS 5          	/* i/o set state flags */
#define IOCS 6          	/* i/o clear state flags */
#define IOFB 0x308      	/* i/o buffer flush */
#define IOFI 0x208      	/* input buffer flush */
#define IOFO 0x108      	/* output buffer flush */
#define IOCE 9          	/* i/o clear error flags */

#define TS_INT28	 1
#define TS_WINOS2	 2
#define TS_NODV 	 4

							/* return count (16bit)	*/
#define RXBC	0x000a		/* get receive buffer count */
#define RXBS	0x010a		/* get receive buffer size */
#define TXBC	0x000b		/* get transmit buffer count */
#define TXBS	0x010b		/* get transmit buffer size */
#define TXBF	0x020b		/* get transmit buffer free space */
#define TXSYNC	0x000c		/* sync transmition (seconds<<8|0x0c) */
#define IDLE	0x000d		/* suspend communication routines */
#define RESUME	0x010d		/* return from suspended state */
#define RLERC	0x000e		/* read line error count and clear */
#define CPTON	0x0110		/* set input translation flag for ctrl-p on */
#define CPTOFF	0x0010		/* set input translation flag for ctrl-p off */
#define GETCPT	0x8010		/* return the status of ctrl-p translation */
#define MSR 	0x0011		/* read modem status register */
#define FIFOCTL 0x0012		/* FIFO UART control */
#define TSTYPE	0x0013		/* Time-slice API type */
#define GETTST	0x8013		/* Get Time-slice API type */

#define SMSMK	0x0014		/* set modem status mask */
#define SMLCR	0x0015		/* set modem line control register */
#define LFN81	0x0315		/* set line format N81 */
#define LFE71	0x1A15		/* set line format E71 */
#define SRXHL	0x001E		/* set receive flow control high limit */
#define SRXLL	0x001F		/* set receive flow control low limit */
#define IGCLS	0x0020		/* input gate close */
#define IGOPN	0xFF20		/* input gate open */


							/* ivhctl() arguments */
#define INT29R 1         	/* copy int 29h output to remote */
#define INT29L 2			/* Use _putlc for int 29h */
#define INT16  0x10      	/* return remote chars to int 16h calls */
#define INTCLR 0        	/* release int 16h, int 29h */

/**********/
/* Macros */
/**********/

#define uchar	unsigned char
#define ushort	unsigned short
#define uint	unsigned int
#define ulong	unsigned long

#define CRLF			{ outchar(CR); outchar(LF); }
#define SYSOP			(useron.level>=90 || sys_status & SS_TMPSYSOP)
#define REALSYSOP		(useron.level>=90)
#define FLAG(x) 		(long)(1L<<(x-'A'))
#define CLS         	outchar(FF)
#define WHERE       	__LINE__,__FILE__
#define SAVELINE		{ slatr[slcnt]=latr; \
							sprintf(slbuf[slcnt<SAVE_LINES ? slcnt++ : slcnt] \
							,"%.*s",lbuflen,lbuf); \
							lbuflen=0; }
#define RESTORELINE 	{ lbuflen=0; attr(slatr[--slcnt]); \
							bputs(slbuf[slcnt]); \
							curatr=lclatr(-1); }
#define SYNC			{ riosync(0); nodesync(); }
#define ASYNC			{ riosync(1); nodesync(); }
#define DCDHIGH 		(sys_status&SS_DCDHIGH || rioctl(IOSTATE)&DCD)

#if DEBUG	/* if DEBUG, call function */
#define DLOG(where,txt) dlog(where,txt)
#else		/* else, do nothing - function isn't even valid */
#define DLOG(where,txt)
#endif

#if defined(__LARGE__) || defined(__HUGE__) || defined(__COMPACT__)
#define MALLOC  farmalloc
#define FREE    farfree
#else
#define MALLOC  malloc
#define FREE    free
#endif

/********************/
/* Type Definitions */
/********************/

/* Note: If you add elements to any of the structures, add them to the END! */
/****************************************************************************/

typedef struct {						/* Users information */
	ushort	number, 					/* Number */
			uls,						/* Number of uploads */
			dls,						/* Number of downloads */
			posts,						/* Number of posts */
			emails,						/* Number of emails */
			fbacks,						/* Number of emails sent to sysop */
			etoday,						/* Emails today */
			ptoday,						/* Posts today */
			timeon,						/* Total time on */
			textra,						/* Extra time for today */
			logons,						/* Total logons */
			ttoday,						/* Time on today */
			tlast,						/* Time on last call */
			ltoday, 					/* Logons today */
			curgrp, 					/* Current group, last on */
			cursub, 					/* Current sub-board */
			curlib, 					/* Current library */
			curdir; 					/* Current directory */
	uchar	level,						/* Security level */
			sex,						/* Sex - M or F */
			rows,               		/* Rows of text */
			alias[LEN_ALIAS+1], 		/* Alias */
			name[LEN_NAME+1],			/* Name - Real */
			handle[LEN_HANDLE+1],		/* Chat handle */
			comp[LEN_COMP+1],			/* Computer type */
			note[LEN_NOTE+1],			/* Public notice about this user */
			address[LEN_ADDRESS+1], 	/* Street Address */
			location[LEN_LOCATION+1],	/* Location of user */
			zipcode[LEN_ZIPCODE+1], 	/* Zip/Postal code */
			pass[LEN_PASS+1],			/* Password - not case sensitive */
			birth[LEN_BIRTH+1], 		/* Birthday in MM/DD/YY format */
			phone[LEN_PHONE+1],			/* Phone number xxx-xxx-xxxx format */
			modem[LEN_MODEM+1],			/* Modem type - 8 chars max */
			netmail[LEN_NETMAIL+1], 	/* NetMail forwarding address */
			xedit,						/* External editor (1 based) */
			leech,						/* Leech attempt counter */
			cmdset, 					/* Command set */
			main_cmd[MAIN_CMDS+1],		/* Custom main command set */
			xfer_cmd[XFER_CMDS+1],		/* Custom transfer command set */
			scan_cmd[SCAN_CMDS+1],		/* Custom Scan command set */
			mail_cmd[MAIL_CMDS+1],		/* Custom E-mail command set */
			comment[LEN_COMMENT+1];		/* Private comment about user */
	ulong	misc,						/* Misc. bits - ANSI, Deleted etc. */
			flags1, 					/* Flag set #1 */
			flags2, 					/* Flag set #2 */
			flags3, 					/* Flag set #3 */
			flags4, 					/* Flag set #4 */
			exempt,						/* Exemption Flags */
			rest,						/* Restriction Flags */
			ulb,						/* Total bytes uploaded */
			dlb,						/* Total bytes downloaded */
			cdt,						/* Credits */
			min,						/* Minutes */
			freecdt;					/* Free credits (renewed daily) */
	time_t	firston,					/* Date/Time first called */
			laston, 					/* Last called */
			expire, 					/* Expiration date */
			pwmod;						/* Password last modified */
			} user_t;

typedef struct {						/* File (transfers) Data */
	uchar   name[13],					/* Name of file FILENAME.EXT */
			desc[LEN_FDESC+1],			/* Uploader's Description */
			uler[LEN_ALIAS+1],			/* User who uploaded */
			opencount,					/* Times record is currently open */
			altpath,					/* Alternate DOS path */
			misc;						/* Miscellaneous bits */
	time_t  date,						/* File date/time */
			dateuled,					/* Date/Time (Unix) Uploaded */
			datedled;					/* Date/Time (Unix) Last downloaded */
	uint    dir,						/* Directory file is in */
			timesdled,					/* Total times downloaded */
			timetodl;					/* How long transfer time */
	long	datoffset,					/* Offset into .DAT file */
			size;						/* Size of file */
	ulong	cdt;						/* Credit value for this file */
			} file_t;

typedef struct {						/* Mail data */
	uchar	title[LEN_TITLE+1], 		/* Title of mail */
			fname[9],					/* Filename (without .MSG ext) */
			from[71],					/* From user */
			to[26], 					/* Dest user */
			flag,						/* Flag - '*'=not read etc. */
			mode;						/* Special bits */
	time_t	time;						/* File date/time */
			} mail_t;

typedef struct {                        /* Message data */
	uchar	*title, 					/* Title of message */
			fname[9],					/* Filename (without .MSG ext) */
			*from,						/* From user */
			*to,						/* To user */
			mode;						/* Special bits */
	time_t	time,						/* Time of post */
			ptr;						/* Pointer to use */
			} post_t;

typedef struct {						/* Node information kept in NODE.DAB */
	uchar	status,						/* Current Status of Node */
			errors,						/* Number of Critical Errors */
			action;						/* Action User is doing on Node */
	ushort	useron, 					/* User on Node */
			connection,					/* Connection rate of Node */
			misc,						/* Miscellaneous bits for node */
			aux;						/* Auxillary word for node */
	ulong	extaux;						/* Extended aux dword for node */
			} node_t;

typedef struct {						/* System/Node Statistics */
	ulong 	logons,						/* Total Logons on System */
			ltoday,						/* Total Logons Today */
			timeon,						/* Total Time on System */
			ttoday,						/* Total Time Today */
			uls,						/* Total Uploads Today */
			ulb,						/* Total Upload Bytes Today */
			dls,						/* Total Downloads Today */
			dlb,						/* Total Download Bytes Today */
			ptoday,						/* Total Posts Today */
			etoday,						/* Total Emails Today */
			ftoday; 					/* Total Feedbacks Today */
	ushort	nusers; 					/* Total New Users Today */
			} stats_t;

typedef struct {						/* FidoNet address */
	ushort	zone,						/* Zone */
			net,						/* Network */
			node,						/* Node */
			point;						/* Point */
			} faddr_t;

typedef struct {                        /* Message sub board info */
	uchar	code[9],					/* Eight character code */
#ifdef SCFG
			read_ar[LEN_ARSTR+1],		/* Access required to read */
			post_ar[LEN_ARSTR+1],		/* Access required to post */
			sname[LEN_SSNAME+1],		/* Short name - used for prompts */
			lname[LEN_SLNAME+1],		/* Long name - used for listing */
			qwkname[11],				/* QWK name - only 10 chars */
            origline[52],               /* Optional EchoMail origin line */
			tagline[81];				/* Optional QWK net tag line */
#else
			*read_ar,
			*post_ar,
			*sname,
			*lname,
			*qwkname,
			*origline,
			*tagline;
#endif
#ifndef SBBS
	uchar	echopath[52];				/* EchoMail path */
#endif
	ushort	maxmsgs,					/* Max number of messages allowed */
			grp,						/* Which group this sub belongs to */
			ptridx; 					/* Index into pointer file */
	ulong	misc;						/* Miscellaneous flags */
	time_t  ptr;                		/* Last Message Read Date */
	faddr_t faddr;						/* FidoNet address */
			} sub_t;

typedef struct {                        /* Message group info */
#ifdef SCFG
	uchar	ar[LEN_ARSTR+1],			/* Access requirements */
			sname[LEN_GSNAME+1],		/* Short name */
			lname[LEN_GLNAME+1];		/* Long name */
#else
	uchar	*ar,
			*sname,
			*lname;
#endif
			} grp_t;

typedef struct {                        /* Transfer Directory Info */
	uchar	code[9],					/* Eight character code */
#ifdef SCFG
			ar[LEN_ARSTR+1],			/* Access Requirements */
			ul_ar[LEN_ARSTR+1], 		/* Upload Requirements */
			dl_ar[LEN_ARSTR+1], 		/* Download Requirements */
			sname[LEN_SSNAME+1],		/* Short name - used for prompts */
			lname[LEN_SLNAME+1],		/* Long name - used for listing */
			path[LEN_DIR],				/* Path to directory for files */
			exts[41],   		        /* Extensions allowed */
#else
			*ar,
			*ul_ar,
			*dl_ar,
			*sname,
			*lname,
			*path,
			*exts,
#endif
			seqdev, 					/* Sequential access device number */
			sort;						/* Sort type */
	ushort	maxfiles,					/* Max number of files allowed */
			lib,						/* Which library this dir is in */
			misc;						/* Miscellaneous bits */
			} dir_t;

typedef struct {                        /* Transfer Library Information */
#ifdef SCFG
	uchar	ar[LEN_ARSTR+1],			/* Access Requirements */
			sname[LEN_GSNAME+1],		/* Short Name - used for prompts */
			lname[LEN_GLNAME+1];		/* Long Name - used for listings */
#else
	uchar	*ar,
			*sname,
			*lname;
#endif
	ushort	offline_dir;				/* Offline file directory */
			} lib_t;

typedef struct {                        /* Gfile Section Information */
	uchar	code[9],					/* Eight character code */
#ifdef SCFG
			ar[LEN_ARSTR+1],			/* Access requirements */
			name[41];           		/* Name of section */
#else
			*ar,
			*name;
#endif
			} txtsec_t;

typedef struct {						/* External Program Information */
	uchar	type,						/* What type of external program */
			event,						/* Execute upon what event */
			main_cmd,					/* Command key to launch from main */
			xfer_cmd,					/* Command key to launch from xfer */
#ifdef SCFG
			ar[LEN_ARSTR+1],			/* Access Requirements */
			run_ar[LEN_ARSTR+1],		/* Run Requirements */
			name[26],					/* Name of External */
			cmd[41],					/* Command line */
			clean[41],					/* Clean-up command line */
			path[41];					/* Start-up path */
#else
			*ar,
			*run_ar,
			*name,
			*cmd,
			*clean,
			*path;
#endif
	uint	misc;						/* Misc. bits - ANSI, DOS I/O etc. */
	ulong	cost;						/* Cost to run in credits */
			} xtrn_t;

typedef struct {						/* External Page program info */
	uchar	misc,						/* Intercept I/O */
#ifdef SCFG
			cmd[41],					/* Command line */
			ar[LEN_ARSTR+1];			/* ARS for this chat page */
#else
			*cmd,
			*ar;
#endif
			} page_t;

typedef struct {						/* Chat action info */
#ifdef SCFG
	uchar	cmd[LEN_CHATACTCMD+1],		/* Command word */
			out[LEN_CHATACTOUT+1];		/* Output */
#else
	uchar	*cmd,
			*out;
#endif
			} chatact_t;

typedef struct {						/* Chat Channel Information */
	uchar	misc,						/* Misc. bits CHAN_* macros */
#ifdef SCFG
			ar[LEN_ARSTR+1],			/* Access requirements */
			name[26],					/* Channel description */
#else
			*ar,
			*name,
#endif
			age;						/* Minimum Age to Access */
	ulong	cost;						/* Cost to join */
			} chan_t;

typedef struct {                        /* Modem Result codes info */
	ushort	code,						/* Numeric Result Code */
			cps,    		            /* Average Transfer CPS */
			rate;   		            /* DCE Rate (Modem to Modem) */
#ifdef SCFG
	uchar   str[LEN_MODEM+1];   		/* String to use for description */
#else
	uchar	*str;
#endif
			} mdm_result_t;

typedef struct {                        /* Transfer Protocol information */
	char    mnemonic,           		/* Letter to select this protocol */
			misc,               		/* Miscellaneous bits */
#ifdef SCFG
			ulcmd[41],          		/* Upload command line */
			dlcmd[41],         			/* Download command line */
			batulcmd[41],       		/* Batch upload command line */
			batdlcmd[41],				/* Batch download command line */
			bicmd[41];					/* Bidirectional command line */
#else
			*ulcmd,
			*dlcmd,
			*batulcmd,
			*batdlcmd,
			*bicmd;
#endif
			} prot_t;

typedef struct {                        /* Extractable file types */
	char    ext[4],             		/* Extension */
#ifdef SCFG
			cmd[41];            		/* Command line */
#else
			*cmd;
#endif
			} fextr_t;

typedef struct {                        /* Viewable file types */
	char    ext[4],            	 		/* Extension */
#ifdef SCFG
			cmd[41];            		/* Command line */
#else
			*cmd;
#endif
			} fview_t;

typedef struct {                        /* Testable file types */
	char    ext[4],             		/* Extension */
#ifdef SCFG
			cmd[41],            		/* Command line */
			workstr[41];        		/* String to display while working */
#else
			*cmd,
			*workstr;
#endif
			} ftest_t;

typedef struct {						/* External Editors */
	char	misc,						/* Misc. bits */
#ifdef SCFG
			name[41],					/* Name (description) */
			lcmd[41],					/* Local command line */
			rcmd[41];					/* Remote command line */
#else
			*name,
			*lcmd,
			*rcmd;
#endif
			} xedit_t;

typedef struct {						/* QWK Network Hub */
	uchar	id[9],						/* System ID of Hub */
			*mode,						/* Mode for Ctrl-A codes for ea. sub */
			node,						/* Node to do the call-out */
			days,						/* Days to call-out on */
#ifdef SCFG
			call[41],					/* Call-out command line to execute */
			pack[41],					/* Packing command line */
			unpack[41]; 				/* Unpacking command line */
#else
			*call,
			*pack,
			*unpack;
#endif
	ushort	time,						/* Time to call-out */
			freq,						/* Frequency of call-outs */
			subs,						/* Number Sub-boards carried */
			*sub,						/* Number of local sub-board for ea. */
			*conf;						/* Conference number of ea. */
#ifndef SCFG
	time_t	last;						/* Last network attempt */
#endif
			} qhub_t;

typedef struct {						/* PCRelay/PostLink Hub */
	uchar	node,						/* Node to do the call-out */
			days,						/* Days to call-out on */
#ifdef SCFG
			name[11],					/* Site Name of Hub */
			call[41];					/* Call-out command line to execute */
#else
			*call;
#endif
	ushort	time,						/* Time to call-out */
			freq;						/* Frequency of call-outs */
#ifndef SCFG
	time_t	last;						/* Last network attempt */
#endif
			} phub_t;

typedef struct {						/* FidoNet msg header */
	uchar	from[36],					/* From user */
			to[36], 					/* To user */
			title[72],					/* Message title */
			time[20];					/* Time in goof-ball ASCII format */
	short	read,						/* Times read */
			destnode,					/* Destination node */
			orignode,					/* Origin node */
			cost,						/* Cost in pennies */
			orignet,					/* Origin net */
			destnet,					/* Destination net */
			destzone,					/* Destination zone */
			origzone,					/* Origin zone */
			destpoint,					/* Destination point */
			origpoint,					/* Origin point */
			re, 						/* Message number regarding */
			attr,						/* Attributes - see FIDO_* */
			next;						/* Next message number in stream */
			} fmsghdr_t;

typedef struct {						/* Alternate command sets */
	char	desc[41],					/* Description */
			menudir[10],				/* Directory off of TEXT\MENU */
			main[MAIN_CMDS+1],			/* Main menu commands */
			xfer[XFER_CMDS+1],			/* Transfer menu commands */
			scan[SCAN_CMDS+1],			/* Message scanning commands */
			mail[MAIL_CMDS+1];			/* E-mail commands */
			} altcmds_t;


typedef struct {
	char 	*openerr,
			*reading,
			*readit,
			*allocerr,
			*error;
			} read_cfg_text_t;

#endif /* Don't add anything after this #endif statement */
